#!/usr/bin/env python3

import argparse
import json
import logging
import os
import sys
import tempfile
from .version import __version__
import gippy

from satstac import Collection, Item, ItemCollection
import satsearch.config as config
from satsearch import Search
from satsearch.cli import SatUtilsParser, main as satsearch
from satfetch import fetch as satfetch


logger = logging.getLogger(__name__)


class SatFetchParser(SatUtilsParser):

    def __init__(self, *args, **kwargs):
        # change defaults from sat-search
        # data directory to store downloaded imagery
        config.DATADIR = os.getenv('SATUTILS_DATADIR', './')
        # filename pattern for saving files
        config.FILENAME = os.getenv('SATUTILS_FILENAME', '${collection}_${date}')
        super().__init__(*args, **kwargs)
        self.download_group.add_argument('--fetch', help='Fetch and clip to AOI', default=None, nargs='*')


def main(items=None, fetch=None, save=None, **kwargs):
    """ Main function for performing a search """
    _save = save if items is None else None
    items = satsearch(items, save=_save, **kwargs)
    # if not downloading nothing more to do
    if fetch is None:
        return
    
    # check that there is a valid geometry for clipping
    feature = items._search.get('parameters', {}).get('intersects', None)
    if feature is None:
        raise Exception('No geometry provided')

    derived_items = []
    # for each date, combine scenes
    for date in items.dates():
        print('Processing files for %s' % date)
        _items = [s for s in items if s.date == date]
        # TODO - split out by user specified metadata (e.g., platform, collection)
        item = satfetch(_items, feature['geometry'], fetch)
        derived_items.append(item)

    # this needs update to sat-stac to support adding metadata to Items
    # see https://github.com/sat-utils/sat-stac/issues/39
    #props = {
    #    'software': 'sat-fetch v%s' % __version__
    #}
    
    col = Collection.create()
    col._data['id'] = 'sat-fetch'
    col._data['description'] = 'Fetch items created by sat-fetch'
    col._data['links'].append({'rel': 'about', 'href': 'https://github.com/sat-utils/sat-fetch'})
    derived_items = ItemCollection(derived_items, collections=[col])
    if save is not None:
        derived_items.save(save)
    return derived_items


def cli():
    parser = SatFetchParser.newbie(description='sat-fetch (v%s)' % __version__)
    args = parser.parse_args(sys.argv[1:])

    cmd = args.pop('command', None)
    if cmd is not None:
        main(**args)


if __name__ == "__main__":
    cli()



import os
import os.path as op

from satsearch import config
from satfetch import fetch, open_image
from satstac import ItemCollection
from unittest import TestCase

testpath = op.dirname(__file__)


class Test(TestCase):
    """ Test main module """

    def get_items(self):
        return ItemCollection.load(op.join(testpath, 'items-landsat.geojson'))

    def test_open_image(self):
        item = self.get_items()[0]
        geoimg = open_image(item, keys=['red'])
        assert(geoimg.nbands() == 1)
        assert(geoimg.xsize() == 7701)

    def test_fetch(self):
        items = self.get_items()
        bands = ['BQA']
        config.DATADIR = ''
        config.FILENAME = os.path.join(testpath, 'test_fetch/${collection}_${date}')
        new_items = fetch(items, items._search.get('parameters', {}).get('intersects', {}), bands)




import json
import logging
import os
import tempfile

from gippy import GeoImage, GeoVector, algorithms as algs
from satstac import Item
from satsearch import config

logger = logging.getLogger(__name__)


OPTS = {'COMPRESS': 'DEFLATE', 'PREDICTOR': '2', 'INTERLEAVE': 'BAND',
        'TILED': 'YES', 'BLOCKXSIZE': '512', 'BLOCKYSIZE': '512'}


def open_image(item, keys=None, nodata=0, download=False):
    """ Open these asset keys from scene as a gippy GeoImage """
    if keys is None:
        keys = item.assets.keys()
    logger.debug('Opening item %s (%s)' % (item.id, ','.join(keys)))
    if download:
        # download items first
        fnames = item.download(keys)
        geoimg = GeoImage.open(fnames)
    else:
        # use GDAL vsi curl driver
        filenames = [item.asset(k)['href'].replace('https:/', '/vsicurl/https:/') for k in keys]
        geoimg = GeoImage.open(filenames, update=False)
    geoimg.set_bandnames(keys)
    geoimg.set_nodata(nodata)
    return geoimg


def geometry_to_GeoVector(geometry):
    # create temporary geometry file
    with tempfile.NamedTemporaryFile(suffix='.geojson', mode='w', delete=False) as f:
        feature = {
            "type": "Feature",
            "geometry": geometry
        }
        fname = f.name
        f.write(json.dumps(feature))
    return GeoVector(fname)


def create_derived_item(items, geometry):
    """ Create metadata for dervied scene from multiple input scenes """
    # data provenance, iterate through links
    links = []
    for i in items:
        selflink = [l['href'] for l in i._data['links'] if i['rel'] == 'self']
        if len(selflink) > 0:
            links.append({
                'rel': 'derived_from',
                'href': selflink[0]
            })
    print(geometry)
    lons = [c[0] for c in geometry['coordinates'][0]]
    lats = [c[1] for c in geometry['coordinates'][0]]
    bbox = [min(lons), min(lats), max(lons), max(lats)]
    # properties
    props = {
        'datetime': items[0]['datetime']
    }
    collections = [item['collection'] for item in items if item['collection'] is not None]
    if len(collections) == 1:
        props['collection'] = collections[0]
    item = {
        'type': 'Feature',
        'id': '%s_%s' % (items[0]['eo:platform'], items[0].date),
        'bbox': bbox,
        'geometry': geometry,
        'properties': props,
        'links': links,
        'assets': {}
    }
    return Item(item)



def fetch(items, geometry, keys, proj=None, res=None):
    """ This fetches data from just the AOI and clips it """
    derived_item = create_derived_item(items, geometry)

    bands = []
    for k in keys:
        bands += items[0].asset(k).get('eo:bands', [])
    filename = items[0].get_filename(path=config.DATADIR, filename=config.FILENAME).replace('.json','.tif')
    os.makedirs(os.path.dirname(filename), exist_ok=True)

    derived_item._data['assets'] = {
        'image': {
            'type': 'image/vnd.stac.geotiff; cloud-optimized=true',
            'title': 'Clipped image',
            'href': filename,
            'eo:bands': bands
        }
    }

    if os.path.exists(filename):
        return derived_item
    try:
        geoimgs = []
        for item in items:
            geoimgs.append(open_image(item, keys))

        # default to first image res and srs
        if res is None:
            res = geoimgs[0].resolution()
            res = [res.x(), res.y()]
        elif len(res) == 1:
            res = [res[0], res[0]]
        if proj is None:
            proj = geoimgs[0].srs()
        geovec = geometry_to_GeoVector(geometry)
        imgout = algs.cookie_cutter(geoimgs, filename, geovec[0], xres=res[0], yres=res[1], 
                                    proj=proj, options=OPTS)
        logger.info("Created %s" % imgout.filename())
        return derived_item
    except Exception as err:
        print('Error: ', str(err))


